@@ -712,29 +712,28 @@ class EditorDialogBase(object):
                 # if the string in not an integer.
                 solfege.win.display_exception_message(e)
         dialog.destroy()
-    def on_save_as(self, widget):
-        """
-        Return True if the file was saved, False if not.
-        """
+    def get_save_as_dialog(self):
         dialog = gtk.FileChooserDialog(_("Save As..."), self,
             gtk.FILE_CHOOSER_ACTION_SAVE,
             (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
              gtk.STOCK_OK, gtk.RESPONSE_OK))
         dialog.set_default_response(gtk.RESPONSE_OK)
+        return dialog
+    def on_save_as(self, widget):
+        """
+        Return True if the file was saved, False if not.
+        """
+        dialog = self.get_save_as_dialog()
         if not os.path.exists(self.savedir):
             try:
                 os.mkdir(self.savedir)
             except OSError, e:
                 pass
-        # If filename is None, then it is a new file, and we will open
-        # the folder in the users home dir.
-        # If the file already have a file name, we will open the folder
-        # containing the file.
-        if not self.m_filename or os.path.isabs(self._get_a_filename()):
-            dialog.set_current_folder(self.savedir)
+        if self.m_filename and os.path.commonprefix([
+               self.savedir, os.path.dirname(self.m_filename)]) == self.savedir:
+            dialog.set_current_folder(os.path.dirname(self.m_filename))
         else:
-            dialog.set_current_folder(os.path.dirname(self._get_a_filename()))
-        dialog.set_current_name(os.path.basename(self._get_a_filename()))
+            dialog.set_current_folder(self.savedir)
         while 1:
             ret = dialog.run()
             if ret == gtk.RESPONSE_OK:
@@ -857,6 +856,18 @@ class ClickableLabel(gtk.LinkButton):
         """
         gtk.LinkButton.__init__(self, label, label)
         self.get_children()[0].set_alignment(0.0, 0.5)
+    def add_heading(self, text):
+        def set_alignment(x, y):
+            for c in self.get_children()[0].get_children():
+                super(gtk.Label, c).set_alignment(x, y)
+        vbox = gtk.VBox()
+        vbox.set_alignment = set_alignment
+        b = gtk.Label()
+        b.set_markup(u"%s" % text)
+        b.set_alignment(0.0, 0.5)
+        vbox.pack_start(b, False)
+        self.get_children()[0].reparent(vbox)
+        self.add(vbox)
     def make_warning(self):
         im = gtk.Image()
         im.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)
