@@ -469,8 +469,8 @@ class FrontPage(ExerciseView):
 class TestsView(ExerciseView):
     def on_link_clicked(self, w, filename):
         solfege.app.test_lessonfile(filename)
-    def display_data(self, data=None, is_search_result=False):
-        self._display_data(data, True, is_search_result)
+    def display_data(self, data=None, is_search_result=False, show_topics=False):
+        self._display_data(data, True, is_search_result, show_topics)
         self.g_searchentry.grab_focus()
     def on_search(self, *button):
         search_for = self.g_searchentry.get_text().lower()
@@ -631,10 +631,12 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
           ('FrontPage', None, _('_Front Page'), 'F5', None,
               lambda w: self.display_frontpage()),
           ('TestsPage', None, _('_Tests Page'), 'F6', None,
-              self.display_testpage),
+              lambda w: self.display_testpage()),
           ('RecentExercises', None, _('_Recent Exercises'), 'F7', None,
               self.display_recent_exercises),
-          ('UserExercises', None, _('_User Exercises'), 'F8', None,
+          ('RecentTests', None, _('_Recent Tests'), 'F8', None,
+              self.display_recent_tests),
+          ('UserExercises', None, _('_User Exercises'), 'F9', None,
               self.display_user_exercises),
           ('HelpHelp', 'gtk-help', _('_Help on the current exercise'), 'F1', None,
             lambda o: solfege.app.please_help_me()),
@@ -983,7 +985,14 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
                    solfege.db.recent(8))])])
         self.display_frontpage(data, show_topics=True)
         self.get_view().g_searchbox.hide()
-    def display_testpage(self, widget=None):
+    def display_recent_tests(self, w):
+        data = frontpage.Page(_('Recent tests'),
+            [frontpage.Column(
+                [frontpage.LinkList(_('Recent tests'),
+                   solfege.db.recent_tests(8))])])
+        self.display_testpage(data, show_topics=True)
+        self.get_view().g_searchbox.hide()
+    def display_testpage(self, data=None, show_topics=False):
         """
         Display the front page of the data  in solfege.app.m_frontpage_data
         """
@@ -991,7 +1000,9 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         if not self.show_view('testspage'):
             self.add_view(TestsView(), 'testspage')
         self.get_view().g_searchbox.show()
-        self.get_view().display_data(solfege.app.m_frontpage_data)
+        if not data:
+            data = solfege.app.m_frontpage_data
+        self.get_view().display_data(data, show_topics=show_topics)
     def on_search_all_exercises(self, widget=None):
         self.set_title("GNU Solfege")
         if not self.show_view('searchview'):
