@@ -335,6 +335,10 @@ class Score:
                             return 1
                         if timepos not in A.m_coldict:
                             return 1
+                        if not A.m_coldict[timepos].m_music:
+                            return 1
+                        if not B.m_coldict[timepos].m_music:
+                            return -1
                         return cmp(B.m_coldict[timepos].m_music.values()[0].m_pitch.semitone_pitch(), A.m_coldict[timepos].m_music.values()[0].m_pitch.semitone_pitch())
                         return 1
                     staff.m_voice_list.sort(f)
@@ -409,7 +413,7 @@ class Score:
                     #
                     if voice.m_coldict[timepos].m_rest:
                         se.append(RestEngraver(timepos, fontsize, 0,
-                              voice.m_coldict[timepos].m_rest.m_duration))
+                              voice.m_coldict[timepos].m_rest))
                         self.m_timeposdict[timepos].m_music \
                            = max(self.m_timeposdict[timepos].m_music, se[-1].get_width())
                 ####################################
@@ -584,13 +588,6 @@ class Staff:
         if timepos not in self.m_coldict:
             self.m_coldict[timepos] = StaffColObj()
             self.w_score().announce_timepos(timepos)
-    def maybe_ledger_lines(self, timepos, ypos):
-        if timepos not in self.m_coldict:
-            self.m_coldict[timepos] = StaffColObj()
-        if self.m_coldict[timepos].m_ledger_up > ypos < -5:
-            self.m_coldict[timepos].m_ledger_up = ypos
-        if self.m_coldict[timepos].m_ledger_down < ypos > 5:
-            self.m_coldict[timepos].m_ledger_down = ypos
 
 
 class Voice:
@@ -732,17 +729,16 @@ def parse_to_score_object(music):
     context = TOPLEVEL
     score = Score()
     chord_duration = None
+    cur_duration = Duration(4, 0)
     tie_is_in_the_air = 0
-    x = len(lexer.m_string)
     times = None
     cur_staff = None
-    while True:
+    for toc, toc_data in lexer:
         try:
-            toc, toc_data = lexer.get()
-        except Exception, e:
-            if 'm_mpd_badcode' not in dir(e):
-                e.m_lineno, e.m_linepos1, e.m_linepos2 = lexer.get_error_location()
-            raise
+            if toc_data.m_duration:
+                cur_duration = toc_data.m_duration
+        except AttributeError:
+            pass
         if toc == Lexer.STAFF:
             assert context == TOPLEVEL
             cur_staff = score.add_staff()
@@ -767,6 +763,8 @@ def parse_to_score_object(music):
         elif toc == Lexer.TRANSPOSE:
             transpose_pitch = toc_data
         elif toc == Lexer.TIME:
+            if not cur_staff:
+                raise ParseError(u"\\time can not be used before \\staff", lexer)
             cur_staff.announce_timepos(timepos)
             cur_staff.add_timesignature(toc_data, timepos)
         elif toc == Lexer.KEY:
@@ -774,6 +772,8 @@ def parse_to_score_object(music):
             if transpose_pitch:
                 p.transpose_by_musicalpitch(transpose_pitch)
             k = (p.get_notename(), toc_data[1])
+            if not cur_staff:
+                raise ParseError(u"\\key can not be used before \\staff", lexer)
             cur_staff.announce_timepos(timepos)
             cur_staff.add_keysignature(timepos, k)
         elif toc == Lexer.TIMES:
@@ -841,6 +841,8 @@ def parse_to_score_object(music):
         elif toc == '~':
             tie_is_in_the_air = 1
         elif toc == Lexer.NOTE and (context in [NOTES, CHORD, START_OF_CHORD]):
+            if not getattr(toc_data, 'm_duration', None):
+                toc_data.m_duration = cur_duration
             if times:
                 toc_data.m_duration.m_tuplet = times
             if context in [NOTES, START_OF_CHORD]:
@@ -848,7 +850,7 @@ def parse_to_score_object(music):
             if relative_mode:
                 toc_data.m_pitch = musicalpitch_relative(
                                           relto, toc_data.m_pitch)
-                relto = toc_data.m_pitch.clone()
+                relto = toc_data.m_pitch
             if transpose_pitch:
                 toc_data.transpose(transpose_pitch)
             cur_voice.add_notehead(timepos, toc_data, stem_dir)
@@ -863,21 +865,29 @@ def parse_to_score_object(music):
                 chord_duration = toc_data.m_duration
                 context = CHORD
         elif toc == Lexer.SKIP and context == NOTES:
+            if toc_data:
+                cur_duration = toc_data
+            else:
+                toc_data = cur_duration
             last_pos = timepos
             timepos = timepos + toc_data.get_rat_value()
         elif toc == Lexer.REST and context == NOTES:
             cur_voice.announce_timepos(timepos)
+            if toc_data:
+                cur_duration = toc_data
+            else:
+                toc_data = cur_duration
             cur_voice.add_rest(toc_data, timepos)
             last_pos = timepos
-            timepos = timepos + toc_data.m_duration.get_rat_value()
+            timepos += toc_data.get_rat_value()
         elif toc == Lexer.STEMDIR:
             stem_dir = toc_data
         elif toc == Lexer.TUPLETDIR:
             tuplet_dir = toc_data
-        elif (toc, toc_data) == (None, None):
-            return score
         else:
             raise ParseError(toc, lexer)
+    return score
+
 
 
 class Lexer:
@@ -923,21 +933,66 @@ class Lexer:
         self.m_notelen = Duration(4, 0)
         self.m_idx = 0
         self.m_last_idx = None
-    def get(self):
-        """
-        return None, None when there are no more tokens.
-        """
+    def __iter__(self):
+        return self
+    @staticmethod
+    def to_string(v):
+        ret = []
+        for toc, toc_data in v:
+            if toc == Lexer.STAFF: # 1
+                ret.append(r"\staff")
+            elif toc == Lexer.VOICE: # 2
+                ret.append(r"\addvoice")
+            elif toc == Lexer.CLEF: # 3
+                ret.append(r"\clef %s " % toc_data)
+            elif toc == Lexer.STEMDIR: # 4
+                ret.append(r"\stem%s " % {const.UP: 'Up',
+                                         const.DOWN: 'Down',
+                                         const.BOTH: 'Both'}[toc_data])
+            elif toc == Lexer.TRANSPOSE: # 5
+                ret.append(r"\transpose %s" % toc_data.get_octave_notename())
+            elif toc == Lexer.TIME: # 6
+                ret.append(r"\time %i/%i " % (toc_data.m_num, toc_data.m_den))
+            elif toc == Lexer.KEY: # 7
+                ret.append(r"\key %s \%s " % toc_data)
+            elif toc == Lexer.NOTE: # 8
+                ret.append("%s%s " % (toc_data.m_pitch.get_octave_notename(),
+                    toc_data.m_duration.as_mpd_string() if toc_data.m_duration else ""))
+            elif toc == Lexer.SKIP: # 9
+                ret.append("s%s " % (toc_data.as_mpd_string() if toc_data else "",))
+            elif toc == Lexer.REST: # 10
+                ret.append("r%s " % (toc_data.as_mpd_string() if toc_data else "",))
+            elif toc == Lexer.RELATIVE: # 11
+                ret.append(r"\relative %s" % toc_data.get_octave_notename())
+            elif toc == Lexer.TIMES: # 12
+                ret.append(r"\times %i/%i{ " % (toc_data.m_num, toc_data.m_den))
+            elif toc == Lexer.TUPLETDIR: # 13
+                ret.append(r"\tuplet%s " % {const.UP: 'Up',
+                                         const.DOWN: 'Down',
+                                         const.BOTH: 'Both'}[toc_data])
+            else:
+                ret.append("%s " % toc)
+        return "".join(ret).strip()
+
+    def next(self):
+        try:
+            return self._next()
+        except _exceptions.MpdException, e:
+            if 'm_mpd_badcode' not in dir(e):
+                e.m_lineno, e.m_linepos1, e.m_linepos2 = self.get_error_location()
+            raise
+    def _next(self):
         # Doing this while loop inside the exception clause is a little
         # faster than using a regular expression.
         try:
             while self.m_string[self.m_idx] in (' ', '\n', '\t'):
                 self.m_idx += 1
         except IndexError:
-            pass
+            raise StopIteration
         self.m_last_idx = self.m_idx
         m = self.re_rest.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             resttype, notelen, dots = m.groups()
             numdots = len(dots)
             if notelen:
@@ -950,21 +1005,20 @@ class Lexer:
                         'goodcode':'%s%i%s' % (resttype, self.m_notelen.m_nh, dots)
                         },
                         self)
-            r = RestRequest(notelen, numdots)
-            if not r.m_duration:
-                r.m_duration = self.m_notelen.clone()
-                self.m_notelen.m_dots = numdots
+            if notelen is 0:
+                retval = None
             else:
-                self.m_notelen = r.m_duration.clone()
-            return self.REST, r
+                self.m_notelen = retval = Duration(notelen, numdots)
+            return self.REST, retval
 
         m = self.re_skip.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             IGN1, skiplen, dots = m.groups()
             numdots = len(dots)
             if skiplen:
                 skiplen = int(skiplen)
+                self.m_notelen = Duration(skiplen, numdots)
             else:
                 skiplen = 0
                 if numdots:
@@ -973,17 +1027,18 @@ class Lexer:
                         'goodcode':'s%i%s' % (self.m_notelen.m_nh, dots)
                         }, lexer)
             if skiplen is 0:
-                retval = self.m_notelen
+                retval = None
             else:
                 self.m_notelen = retval = Duration(skiplen, numdots)
             return self.SKIP, retval
         m = self.re_melodic.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             notename, IGN1, IGN2, notelen, dots = m.groups()
             numdots = len(dots)
             if notelen:
                 notelen = int(notelen)
+                self.m_notelen = Duration(notelen, numdots)
             else:
                 notelen = 0
                 if dots:
@@ -992,59 +1047,55 @@ class Lexer:
                         'goodcode':'%s%i%s' % (notename, self.m_notelen.m_nh, dots)
                         }, self)
             n = MusicRequest(notename, notelen, numdots)
-            if not n.m_duration:
-                n.m_duration = self.m_notelen.clone()
-            else:
-                self.m_notelen = n.m_duration.clone()
             return self.NOTE, n
         m = self.re_staff.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             return self.STAFF, None
         m = self.re_voice.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             return self.VOICE, None
         m = self.re_relative.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             return self.RELATIVE, MusicalPitch.new_from_notename(m.group(1))
         m = self.re_clef_quoted.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             return self.CLEF, m.group(1)
         m = self.re_clef.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             return self.CLEF, m.group(1)
         m = self.re_stem_updown.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             d = [const.UP, const.DOWN, const.BOTH][['Up', 'Down', 'Both'].index(m.group(2))]
             return self.STEMDIR, d
         m = self.re_tuplet_updown.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             d = [const.UP, const.DOWN, const.BOTH][['Up', 'Down', 'Both'].index(m.group(2))]
             return self.TUPLETDIR, d
         m = self.re_transpose.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             return self.TRANSPOSE, MusicalPitch.new_from_notename(m.group(1))
         m = self.re_time.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             return self.TIME, TimeSignature(int(m.group(1)), int(m.group(2)))
         m = self.re_key.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             return self.KEY, (m.group(1), m.group(2))
         m = self.re_times.match(self.m_string, self.m_idx)
         if m:
-            self.m_idx += len(m.group())
+            self.m_idx = m.end()
             return self.TIMES, Rat(int(m.groups()[0]), int(m.groups()[1]))
         if self.m_idx == len(self.m_string):
-            return None, None
+            raise StopIteration
         self.m_idx += 1
         return self.m_string[self.m_idx-1], None
     def get_error_location(self):
@@ -1066,6 +1117,19 @@ class Lexer:
         while line_end < len(self.m_string) and self.m_string[line_end] != '\n':
             line_end += 1
         return (lineno, self.m_last_idx - line_start, self.m_idx - line_start)
+    def set_first_pitch(self, pitch):
+        """
+        Modify the first pitch of the music.
+        """
+        assert isinstance(pitch, MusicalPitch)
+        for toc, toc_data in self:
+            if toc == Lexer.NOTE:
+                self.m_string = u" ".join(
+                    [x for x in [
+                    self.m_string[:self.m_last_idx].rstrip(),
+                    "%s%s" % (pitch.get_octave_notename(), toc_data.m_duration.as_mpd_string() if toc_data.m_duration else ""),
+                    self.m_string[self.m_idx:].lstrip()] if x])
+                break
 
 
 def validate_only_notenames(s):
@@ -1074,13 +1138,12 @@ def validate_only_notenames(s):
     (pitch and duration). No ties or other tokens are allowed.
     """
     lex = Lexer(s)
-    while 1:
-        try:
-            toc, toc_data = lex.get()
-        except InvalidNotenameException:
-            return lex.get_error_location()
-        if not toc:
-            break
-        if toc != lex.NOTE:
-            return lex.get_error_location()
+    try:
+        for toc, toc_data in lex:
+            if not toc:
+                break
+            if toc != lex.NOTE:
+                return lex.get_error_location()
+    except InvalidNotenameException:
+        return lex.get_error_location()
     return None, None, None
