@@ -1,5 +1,7 @@
 #!/bin/sh
-PYTHON=/c/Python25/python.exe
+PYVER=26
+PYTHON=/c/Python$PYVER/python.exe
+
 build() {
   # MS Windows have other defaults than linux. The 'sed' I have installed on
   # my windows machine don't support the -i option.
@@ -7,19 +9,19 @@ build() {
   sed -e "s/type=external-midiplayer/type=sequencer-device/" -e "s/csound=csound/csound=AUTODETECT/" -e "s/mma=mma/mma=AUTODETECT/" tmp.cfg > default.config
   rm tmp.cfg
 
-  ./configure PYTHON=/c/Python25/python.exe --disable-pygtk-test --enable-winmidi
-  make skipmanual=yes PYTHON_INCLUDES=-I/c/Python25/include
+  ./configure PYTHON=/c/Python$PYVER/python.exe --disable-pygtk-test --enable-winmidi
+  make skipmanual=yes PYTHON_INCLUDES=-I/c/Python$PYVER/include
   make winbuild
 }
 
 prepare_installer() {
   rm win32 -rf
-  make DESTDIR=win32 prefix="" install skipmanual=yes PYTHON_INCLUDES=-I/c/Python25/include
+  make DESTDIR=win32 prefix="" install skipmanual=yes PYTHON_INCLUDES=-I/c/Python$PYVER/include
   mkdir win32/python
-  (cd win32/python && tar zxf ../../../my-python-2.5.tgz)
+  (cd win32/python && tar zxf ../../../my-python-26.tgz)
   $PYTHON tools/get-gtk-files.py bin
   $PYTHON tools/get-gtk-files.py unpack
-  mv win32/zlib1.dll win32/bin
+  mv win32/zlib-1.2.4/zlib1.dll win32/bin
   cp -a ../pygtk-stuff/* win32/python/lib/site-packages
   (cp solfege/soundcard/winmidi.pyd win32/share/solfege/solfege/soundcard)
   cp win32-start-solfege.pyw win32/bin
@@ -36,9 +38,9 @@ prepare_installer() {
 mk_pygtk_stuff() {
   rm -rf ../pygtk-stuff
   mkdir ../pygtk-stuff
-  cp /C/Python25/Lib/site-packages/pygtk.* ../pygtk-stuff/
-  cp -a /C/Python25/Lib/site-packages/cairo/ ../pygtk-stuff/
-  cp -a /C/Python25/Lib/site-packages/gtk-2.0/ ../pygtk-stuff/
+  cp /C/Python$PYVER/Lib/site-packages/pygtk.* ../pygtk-stuff/
+  cp -a /C/Python$PYVER/Lib/site-packages/cairo/ ../pygtk-stuff/
+  cp -a /C/Python$PYVER/Lib/site-packages/gtk-2.0/ ../pygtk-stuff/
 }
  
 if test "x$1" = "xbuild"; then
